/* GKrellM
|  Copyright (C) 1999-2000 Bill Wilson
|
|  Author:  Bill Wilson    bill@gkrellm.net
|  Latest versions might be found at:  http://gkrellm.net
|
|  This program is free software which I release under the GNU General Public
|  License. You may redistribute and/or modify this program under the terms
|  of that license as published by the Free Software Foundation; either
|  version 2 of the License, or (at your option) any later version.
|
|  This program is distributed in the hope that it will be useful,
|  but WITHOUT ANY WARRANTY; without even the implied warranty of
|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
|  GNU General Public License for more details.
| 
|  To get a copy of the GNU General Puplic License,  write to the
|  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Changes: 
 * v1.2 March 3 
 *         actually display fan speed using  krell-slider ;) 
 *         also changed label text style to CLOCK_STYLE,
 *         and postion to MON_MAIL. Should make a lot of options configurable rather than hard-
 *         coded values.
 *
 * v1.1 February 
 *         updated to work with gkrellm v0.9.0>
 *
 * v1.0 Beginning of 2000
 *        working version
 */

/*
 * plugin_fan.c v1.2
 * Requires gkrellm v0.9.1 or greater.
 *
 * GKrellm plugin to show fan speeds. 
 * Requires lm_sensors installed and working.
 * Reads fan speed from PROC_SENSORS_DIR/chip/fanX ( X e 1,2,3..)
 *
 * Basically its sensors.c converted to plugin with modifications
 * to read fan-speeds instead of temperatures.
 * 
 * Quick hack to learn how to code gkrellm plugins and stuff.
 *
 * Code is ugly but its works! ;)
 *
 * Please send ideas & fixups to <jappe@iki.fi>
 * Made by Jarkko Lietolahti <jappe@iki.fi>
 */

 /* If this file is compiled as a shared object file and installed in
|  in your ~/.gkrellm/plugins directory, then it will be loaded when
|  gkrellm starts up.
|     gcc -fPIC `gtk-config --cflags` `imlib-config --cflags-gdk` -c template.c
|     gcc -shared -Wl -o template.so template.o
|     cp template.so ~/.gkrellm/plugins
*/


#include "gkrellm.h"
#include <dirent.h>

#define   CONFIG_SAVE_KEYWORD "sensors_fan"
#define   PROC_SENSORS_DIR	"/proc/sys/dev/sensors"

static Panel *plug;
static gint sensor_fan_files;
static GList	*fan_file_glist	= NULL;
static GList      *fan_names_glist = NULL;

typedef struct
{
  gchar		*name;
  gchar		*temp_file;
  gfloat		factor;			/* Scale sensor tempx reading	*/
  gfloat		offset;			/* Add to sensor tempx reading	*/

  GtkWidget	*entry;
  GtkWidget	*factor_spin_button;
  GtkWidget	*offset_spin_button;
}
Sensor;

static gint plug_expose_event(GtkWidget *widget, GdkEventExpose *event)
{
  if( GK.debug )  
  	fprintf( stderr, "plug_expose_event\n" );
  	
  if (widget == plug->drawing_area)
	{
	  gdk_draw_pixmap(widget->window,
					  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
					  plug->pixmap,
					  event->area.x, event->area.y,
					  event->area.x, event->area.y,
					  event->area.width, event->area.height);
	}
  return FALSE;
}

gint read_rpm(gchar *name)
{
  FILE	*f;
  GList	*list;
  Sensor	*sensor;
  gfloat	temp	= 0;

  if( GK.debug )
	fprintf( stderr, "read_rpm for %s\n", name );

  for (list = fan_file_glist; list; list = list->next)
	{
	  sensor = (Sensor *) list->data;
	  if( GK.debug ) 
		fprintf( stderr, "sensor name=%s\n", sensor->name );
	  if (sensor->name && strcmp(sensor->name, name) == 0)
		{
		  if( GK.debug )
			fprintf( stderr, "about to open file %s\n", sensor->temp_file );
		  if ((f = fopen(sensor->temp_file, "r")) != NULL)
			{
			  fscanf(f, "%*f %f", &temp);
			  fclose(f);
			  temp = temp * sensor->factor + sensor->offset;
			  break;
			}
		}
	}
  return (int)temp;
}

static void update_plugin() {
  Panel *p;
  Krell *krell;
  static GList *name;
  static gint max_rpm = 0;
  gchar buf[16];
  gint rpm;
  
  if( !name ) name = fan_names_glist;

  if( GK.second_tick ) {
	p = plug;
	krell = KRELL(plug);
	if( !name ) { 
	  // there're no fans configured
	  sprintf( buf, "no fans");
	  if( GK.debug ) 
		fprintf( stderr, "buf=%s\n", buf );
	  p->label->string = buf;
	  gkrellm_draw_panel_label( plug,GK.bg_panel_image[DISK_STYLE]);
	  gkrellm_draw_layers( plug );
	  
	} else { 
	  // and there are fans..
	  if( GK.debug ) 
		fprintf( stderr, "update_plugin with name=%s\n", (gchar*)name->data);
	  if( (rpm=read_rpm( (gchar*)name->data)) == 0 ) {
		if( GK.debug )
		  fprintf( stderr, "ops: read_rpm returned  0.\n");
		return;
	  }
	  sprintf( buf,"%s:%d ",  (gchar*)name->data, rpm );
		
	  // 1st: update text label
	  if( GK.debug ) 
		fprintf( stderr, "buf=%s\n", buf );
	  p->label->string = buf;
	  gkrellm_draw_panel_label( plug,GK.bg_panel_image[CLOCK_STYLE]);

	  // 2nd: update krell
	  if( rpm > max_rpm ) max_rpm= rpm;
	  krell->full_scale = max_rpm + 0.25*max_rpm;
	  krell->previous = 0;
	  gkrellm_update_krell( plug, KRELL(plug), rpm  );

	  // 3rd: update updates ;)
	  gkrellm_draw_layers( plug );

	  if( !name->next )  name = fan_names_glist; 
	  else  name = name->next;
	} // EOFans
  } // five_second_tick
} // EOF

static void create_plugin(GtkWidget *vbox, gint first_create) {
  Style			*plug_style;
  GdkImlibImage	*bg_image, *krell_image;
  

  /* Allocate a panel structure.  plug->label and plug->textstyle
	 |  structures will also be allocated.
  */
  if( first_create ) {
	plug = gkrellm_panel_new0();
  } else {
	gkrellm_destroy_decal_list(plug);
  }
  
  /* Allocate a local Style structure and copy the theme configured
	 |  style so I can make a local change to label_position.  If I had
	 |  no changes, I could just plug_style = gkrellm_meter_style(MY_STYLE);
  */
   plug_style = gkrellm_style_new0();
  *plug_style = *gkrellm_meter_style(DEFAULT_STYLE);;
  plug_style->label_position = LABEL_CENTER;
  
  
  /* Create a krell.  A Krell structure is allocated and linked into
	 |  the list of krells pointed to by plug->krell.
  */
   krell_image = gkrellm_krell_meter_image(DEFAULT_STYLE);
   gkrellm_create_krell( plug, krell_image, plug_style  );

  /* Initialize the panel textstyle to TEXTSTYLE_TIME default and allow
	 |  gkrellmrc styles to override.
  */

  //  gkrellm_default_textstyle(plug->textstyle, TEXTSTYLE_TIME, plug_style);
  plug->textstyle = gkrellm_meter_textstyle(  DEFAULT_STYLE );
  /* Configure panel calculates the panel height needed for the label
	 |  supplied.  The height is adjusted based on the borders specified
	 |  in the style and the heights of any decals or krells that have been
	 |  set up for the panel.
  */
  gkrellm_configure_panel(plug, "Fan", plug_style);

  /* Build the configured panel with a background image and pack it into
	 |  the vbox assigned to this monitor.
  */
  bg_image = gkrellm_bg_meter_image(DEFAULT_STYLE) ;
  gkrellm_create_panel(vbox, plug, bg_image);

  /* GKrellM has to know how high the panel is so it can render the side
	 |  frames to the right height.
  */
  gkrellm_monitor_height_adjust(plug->h+5);

  if( first_create ) { 
	gtk_signal_connect(GTK_OBJECT (plug->drawing_area), "expose_event",
					   (GtkSignalFunc) plug_expose_event, NULL);
  }
}





static gchar *
chip_sensor(gchar *sensor_file)
{
  gchar	*s;

  s = sensor_file + strlen(PROC_SENSORS_DIR);
  if (*s == '/')
	return s + 1;
  return "";
}

/* Associate a name (fan1,fan2,fan3, ...) with chip/temperature
  |  file (eg w83782d-i2c-0-2d/fan1).
  */
static Sensor *
link_temperature_name(gchar *name, gchar *temp)
{
  GList	*list;
  Sensor	*sensor;
  gchar	*s;
  
  if( GK.debug ) 
	fprintf( stderr, "name=%s temp=%s\n", name, temp  );
  
  for (list = fan_file_glist; list; list = list->next)
	{
	  sensor = (Sensor *) list->data;
	  if (sensor->temp_file)
		{
		  s = chip_sensor(sensor->temp_file);
		  if( GK.debug ) 
			fprintf( stderr, "s=%s temp=%s\n", s, temp  );
		  if (*s != '\0' && strcmp(s, temp) == 0)
			{
			  if( GK.debug ) 
				fprintf( stderr, "sensor->name=%s\n", sensor->name );
			  if (sensor->name)
				g_free(sensor->name);
			  sensor->name = g_strdup(name);
			  if( GK.debug ) 
				fprintf( stderr, "after:sensor->name=%s\n", sensor->name );
			  fan_names_glist = g_list_append(fan_names_glist , sensor->name );
			  return sensor;
			}
		}
	}
  return NULL;
}


static void scan_for_temp_files(gchar *chip_dir)
{
  DIR				*dir;
  struct dirent	*dentry;
  Sensor			*sensor;
  gchar			temp_file[256];
  
  if ((dir = opendir(chip_dir)) == NULL)
	return;
  while ((dentry = readdir(dir)) != NULL)
	{
	  if (dentry->d_name[0] != '.' && dentry->d_ino > 0)
		{
		  if (strncmp(dentry->d_name, "fan", 3) == 0) // JL
			{
			  sprintf(temp_file, "%s/%s", chip_dir, dentry->d_name);
			  sensor = g_new0(Sensor, 1);
			  sensor->temp_file = g_strdup(temp_file);
			  sensor->name = g_strdup("");			/* Simplify free code */
			  sensor->factor = 1.0;
			  sensor->offset = 0;
			  fan_file_glist = g_list_append(fan_file_glist, sensor);
			  ++sensor_fan_files;
			}
		}
	}
  closedir(dir);
}

static void register_fan() {
  DIR               *dir;
  struct dirent	*dentry;
  gchar			chip_dir[256];
  
  sensor_fan_files = 0;
  if ((dir = opendir(PROC_SENSORS_DIR)) == NULL)
	return;
  while ((dentry = readdir(dir)) != NULL)
	{
	  if (dentry->d_name[0] != '.' && dentry->d_ino > 0)
		{
		  sprintf(chip_dir, "%s/%s", PROC_SENSORS_DIR, dentry->d_name);
		  if (isdir(chip_dir)) 
			scan_for_temp_files(chip_dir);
			
		}
	}
  closedir(dir);
}


static void
save_fan_config(FILE *f)
{
  GList	*list;
  Sensor	*s;
  gfloat	factor;

  for (list = fan_file_glist; list; list = list->next)
	{
	  s = (Sensor *) list->data;
	  if (s->name && *(s->name) != '\0')
		{
		  if ((factor = s->factor) < 0.1)
			factor = 1.0;
		  fprintf(f, "%s %s %s %f %f\n", CONFIG_SAVE_KEYWORD,s->name,
				  chip_sensor(s->temp_file), factor, s->offset);
		}
	}
}

static void
load_fan_config(gchar *arg)
{
  gchar	*name, *chip, *factor, *offset;
  Sensor	*s;
  gchar	buf[96];

  strcpy(buf, arg);
  name = strtok(buf, " \t\n");
  if (name == NULL)
	return;
  chip = strtok(NULL, " \t\n");

  if (   chip == NULL
		 || (s = link_temperature_name(name, chip)) == NULL
		 )
	return;

  offset = NULL;
  factor = strtok(NULL, " \t\n");
  if (factor)
	offset = strtok(NULL, " \t\n");
  s->factor = factor ? atof(factor) : 1.0;
  s->offset = offset ? atof(offset) : 0;
}


/* ------------------------------------------------------------------- */
/* Apply sensor tab widget values to the UC and to fan_file_glist entries.
   |  apply_stat_config() will handle applying what is updated here to the
   |  cpu and proc panels.
  */
static void
apply_fan_config()
{
  GList	*list;
  Sensor	*s;
  gchar	*name;
  gfloat	factor, offset;

  if (!sensor_fan_files)
	return;
  for (list = fan_file_glist; list; list = list->next)
	{
	  s = (Sensor *) list->data;
	  name = gtk_entry_get_text(GTK_ENTRY(s->entry));
	  factor = gtk_spin_button_get_value_as_float(
												  GTK_SPIN_BUTTON(s->factor_spin_button));
	  offset = gtk_spin_button_get_value_as_float(
												  GTK_SPIN_BUTTON(s->offset_spin_button));
	  if (strcmp(s->name, name) != 0)
		{
		  if (s->name)
			g_free(s->name);
		  s->name = g_strdup(name);
		}
	  if (s->factor != factor)
		s->factor = factor;
	  if (s->offset != offset)
		s->offset = offset;
	}
}

static void
create_fan_tab(GtkWidget *vbox)
{
  GtkWidget		*table;
  GtkWidget		*spinbutton;
  GtkSpinButton	*spin;
  GtkAdjustment	*adj;
  GtkWidget		*separator;
  GtkWidget		*label;
  GtkWidget		*entry;
  GList			*list;
  Sensor			*s;
  gchar			buf[256];
  gchar			*string;
  gint			i;

  table = gtk_table_new (sensor_fan_files + 2, 4, FALSE /*homogeneous*/);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 3);

  label = gtk_label_new("Sensor Label");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);

  label = gtk_label_new(PROC_SENSORS_DIR);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
  label = gtk_label_new("Fan Files"); //
  gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 1, 2);

  label = gtk_label_new("Fan Correction"); //
  gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 4, 0, 1);
  label = gtk_label_new("Factor");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 1, 2);
  label = gtk_label_new("Offset");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 3, 4, 1, 2);

  for (i = 2, list = fan_file_glist; list; list = list->next, ++i)
	{
	  s = (Sensor *) list->data;

	  entry = gtk_entry_new_with_max_length(8);
	  s->entry = entry;
	  if (s->name)
		gtk_entry_set_text(GTK_ENTRY(entry), s->name);
	  gtk_widget_set_usize(entry, 36, 0);
	  gtk_table_attach_defaults(GTK_TABLE(table), entry, 0, 1, i, i + 1);

	  label = gtk_label_new(chip_sensor(s->temp_file));
	  gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, i, i + 1);

	  adj = (GtkAdjustment *) gtk_adjustment_new(s->factor,
												 0.5, 2.0, 0.01 /*step*/, 0.1 /*page*/, 0.0);
	  spinbutton = gtk_spin_button_new (adj, 0.5, 2 /*digits*/);
	  s->factor_spin_button = spinbutton;
	  gtk_widget_set_usize(spinbutton, 36, 0);
	  spin = GTK_SPIN_BUTTON(spinbutton);
	  gtk_spin_button_set_numeric(spin, TRUE);
	  gtk_table_attach_defaults(GTK_TABLE(table), spinbutton, 2, 3, i, i+1);

	  adj = (GtkAdjustment *) gtk_adjustment_new(s->offset,
												 -50.0, 50.0, 1.0 /*step*/, 5.0 /*page*/, 0.0);
	  spinbutton = gtk_spin_button_new (adj, 0.5, 0 /*digits*/);
	  s->offset_spin_button = spinbutton;
	  gtk_widget_set_usize(spinbutton, 36, 0);
	  spin = GTK_SPIN_BUTTON(spinbutton);
	  gtk_spin_button_set_numeric(spin, TRUE);
	  gtk_table_attach_defaults(GTK_TABLE(table), spinbutton, 3, 4, i, i+1);
	}

  string = "\"fan1\", \"fan2\"..., or \"fan3\"";

  sprintf(buf,
		  "Sensor Labels can be %s for fans \n" \
		  "which will show on the GKrellM panel. \n", 
		  string);

  label = gtk_label_new(buf);
  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);

}


static Monitor	monitor_fan =
{
  "Fan",				/* Name, for config tab.	*/
  0,					/* Id,  0 if a plugin		*/
  create_plugin,   		/* The create function		*/
  update_plugin,	  	/* The update function		*/
  create_fan_tab,	       /* The config tab create function	*/
  apply_fan_config,        /* Apply the config function		*/

  save_fan_config,         /* Save user conifg			*/
  load_fan_config,          /* Load user config			*/
  CONFIG_SAVE_KEYWORD,			/* config keyword			*/

  NULL,				/* Undef 2	*/
  NULL,				/* Undef 1	*/
  NULL,				/* Undef 0	*/

 MON_MAIL,		  	/* insert_before_id - place plugin before this mon */

  NULL,				/* Handle if a plugin, filled in by GKrellM		*/
  NULL				/* path if a plugin, filled in by GKrellM		*/
};

Monitor *
init_plugin(void)
{
  if( GK.debug ) 
	fprintf( stderr, "init_fan_monitor\n");
  register_fan();
  if (!sensor_fan_files)
	monitor_fan.create_config = NULL; 
  return &monitor_fan;
}

